
#include "stdio.h"
#include "math.h"
#include <stdlib.h>
#include <time.h>

#define ulong unsigned long // To generate pseudo-random numbers with KISS
#define RAND_MAX_KISS ((unsigned long) 4294967295)

#define	DMax 60			// Max number of dimensions of the search space
#define fMax 6			// Max number of constraints +1
#define RMax 101		// Max number of runs
#define	SMax 100		// Max swarm size
#define stratMax 3 		// Maximum number of different strategies
#define zero  1.0e-30	// To avoid numerical instabilities with some compilers
#define infinity 1.0e30
#define NMax 1000		// Max number of points when the landscape is read on a file
#define ClassMax 50 	// Max number of points for the curve "success rate vs FEmax"

#define ERROR(a) {printf("\nERROR: %s\n", a);exit(-1);}

// EXPERIMENTAL! For variograms and kriging
#define MMax 500 // Max size of a "memory" list
#define LMax 500 // Max size of a local position list. Should be >= DMax+1
// Must be <= MMax
#define NMaxVario 502 // Should be MMax + 2
#define CMax 4  //  Max number of coefficients (parameters) in a model of variogram
#define kMax 10 //  Max number classes of distance +1
#define modelMax 4 // Max number of models
#define valueNbMax 100

//-------------------- Structures
struct quantum 
{
	double q[DMax];
	int size; 
};

struct SS 
{ 
	int D;
	double max[DMax];  	// Initialisation, and feasible space
	double maxInit[DMax];
	double min[DMax]; 
	double minInit[DMax];
	struct quantum q;		// Quantisation step size. 0 => continuous problem
	double maxS[DMax];	// Search space
	double minS[DMax];
	double maxR[DMax];  // To save the real search space if the option "normalise"
	// is activated
	double minR[DMax];
	int valueNb; 		 	// if >0, the search space is a list of "valueNb"values.
};

struct param 
{
	int aroundNb; 		// Nb of points "around" a given one, for "smoothed" evaluation
	double c1;			// Confidence coefficient
	int C1;					// Option. How it may be defined and modified
	double c2;			// Idem
	int C2;					
	int clamping;			// Position clamping or not
	int K;				// Max number of particles informed by a given one
	int choose_LEA;		// Define the kind of Local Exploitation Area
	double distPow; 		// For decreasing random distribution coefficient  (R>=2)
	int draw_LEA;			// How a point is sampled (drawn) in the LEA
	double force_LEA;		// Whether a particle is forced to a LEA or not
	int explOption;		// How size_LEA is modified during the run
	int explSave;			// Flag to save or not the exploitation rate on a file
	double inheritance; // A probability. A particle may inherit the parameters
											// w, c1, and c2 from its best neighbour 
	int init;				// How the position is initialised
	int initLink; 		// What kind of re-initialisation for the links
	int initLinkNbMax;
	int initVel;			// How the velocity is initialised 
	double mutation[3]; 	// Probability of mutation for parameters w,c1,c2
	
	double NisB;				// In [0,1]. If >0, apply a "nearer is better" strategy. 
											// See paramBPSO for details

	int normalise;		// 1 => the search space is "normalised" as [0,1]^D
	int nonLinear; 		// Non linear option for the velocity update equation
	double nonLinearCoeff[3];	
	int opposite;			// Experimental. One may take the "opposite" of a particle
	double p;				// Probability threshold for random topology	
	// (is actually computed as p(S,K) )
	int quant;			// Option for adaptive quantisation
	int R; 				// Kind of random distribution R
	int rand; 			// Kind of randomness (KISS or computer dependent)
	int randOrder;		// Random choice of particles or not
	int S;				// Swarm size
	double scout;		// Rate of scout particles in [0,1] (for strat=4)
	double select;	// Proportion of bad particles that are reinitialised
	double sigma; 		// Standard deviation if Gaussian distribution (R=1)
	double size_LEA;		// Relative size of the LEA
	int stop;				// Flag for stop criterion
	double w[stratMax];	// "weight" coefficient for the velocity
	int W;						// Option. How it may be defined and modified
	double w0Rate; 		// Rate of particles using local strategy 0 (when param.strat=1)	
};

struct fitness 
{
	int size; 
	double f[fMax];
};

struct position 
{ 
	struct fitness f;				// Fitness value  + constraints <=0
	int improved;			// Has been improved or not   
	int size;				// Number of dimensions  D
	double x[DMax];		// Coordinates
	int forced;			// Has been forced or not
	double d; 			// Distance to a given point
};


struct velocity 
{  
	int size;				// Number of dimensions D  
	double v[DMax]; 		// Components
};

struct problem 
{ 
	int constraint;			// Number of constraints
	double epsilon; 		// Admissible error
	double epsConstr;  // Admissible error for each constraint
	int evalMax; 			// Maximum number of fitness evaluations
	int function; 		// Function code
	int nb;				// Number of points for the "repartition on nb points" problem (code -1)
	double objective; 	// Objective value					
	struct position solution; // Solution position (if known, just for tests)	
	struct SS SS;			// Search space
};

struct swarm 
{ 
	int best; 				// Rank of the best particle
	struct position P[SMax];	// Previous best positions found by each particle
	struct position Pp[SMax]; // Previous previous best (for information)
	int lBest[SMax]; 			// Flag: lBest[s]=1 means Pp[s] was a local best
	int S; 					// Swarm size 
	struct velocity V[SMax];	// Velocities
	struct position X[SMax];	// Positions
	struct position XPrev[SMax]; // Previous positions
	double w[SMax];			// "weight", for each particle
	double c1[SMax];			// Confidence coefficient, for each particle
	double c2[SMax];			// Confidence coefficient, for each particle
	int R[SMax];				// Distribution R, for each particle
	int worst;				// Rank of the worst particle
};

struct result 
{  
	double nEval; 			// Number of evaluations  
	struct swarm SW;			// Final swarm
	struct fitness error;				// Numerical result of the run
	double convRate; 			// Ror information: how fast was the convergence
};

struct sF{int s; double f;}; // To sort the positions according to the fitness

struct lBest {int rank; double x[DMax];}; // To sort the local bests

struct xv // For move and confinement subprograms
{
	struct position x; 
	struct velocity v;
}; 

// ---- Specific to LEA (Local exploitation areas)
// rank[] = flags. rank[i]=1 means "particle i is a local best"
// ex[] = search space "around" each local best
struct exploit {int rank[SMax]; struct SS exI[SMax];double rho;};
struct posList {int size; struct position p[LMax];}; 
// Spefific to some uses  
struct landscape {int N;double x[NMax];double fx[NMax];}; // 1D-landscape

// EXPERIMENTAL. Specific to variograms and Kriging 
struct model {int model;int size;double c[CMax];double error;}; // For models (variograms)
struct swarmModel
{ 
	int best; 				// rank of the best particle
	struct model P[SMax];		// Previous best positions found by each particle
	// "model" is not used here
	int S; 					// Swarm size 
	struct velocity V[SMax];	// Velocities
	struct model X[SMax];		// Positions 
};
struct matrix {int size; long double m[kMax+1][kMax+1];};
struct vector {int size; long double v[kMax+1];};
struct GammaC {struct matrix G;struct model c;};
struct krigin {
	struct GammaC Gamma;
	struct position M[MMax]; // Memorise positions
	int MRank,MSize;
	double xMax[DMax],xMin[DMax];
};

// -------------------- Sub-programs
ulong	rand_kiss(); // For the pseudo-random number generator KISS
void	seed_rand_kiss(ulong seed); // "

double alea (double a, double b);
int alea_integer (int a, int b);
double alea_normal (double mean, double stdev);
struct position alea_sphere(int D, struct position center, double radius1,double radius2,
							int randType, double randGamma);
int betterThan(struct fitness f1, struct fitness f2);
static int compareDist (void const *a, void const *b); // For qsort
struct xv confinement(struct xv xv,
    struct problem pb,struct param param,int R);
struct fitness constraint(struct position x, int functCode, double eps);
struct position discrete(struct position pos, struct problem pb, int quant,
    struct quantum qAdapt);
double distanceL(struct position x1, struct position x2, double L);
//struct velocity aleaVector(int D,double coeff); // FOR FUTURE USE
//double normL (struct velocity v,double L);	// FOR FUTURE USE

struct position hammersley(struct SS SS, int S, int deb, int s);
struct position initPos(int init, int SSize, struct problem pb);
int initDistrib(int R);
double initW(struct param param);
double initC1(struct param param);
double initC2(struct param param);

struct velocity initVel(int initVel, int SSize, struct problem pb, 
    struct position pos[],int s0);
    
double max(double a,double b);
double min(double a,double b);

struct fitness perf (struct position x, struct problem pb, int option, int aroundNb, 
    int S,int normalise);	// Fitness evaluation
struct position quantis (struct position x, struct SS SS);
struct problem problemDef(int functionCode, FILE *data);
struct quantum quantAdapt(struct swarm SW, struct problem pb, int quant);
struct result PSO ( struct param param, struct problem problem, int level);
int sign (double x);

struct swarm swarmSelect(struct swarm SW0, double select, struct problem pb);

struct xv move(struct result R,
    int s, int g, struct problem pb, struct param param);

// Subroutines for local search
struct xv drawLEA(struct param param, struct swarm SW, int s0, int g, 
    struct exploit ex, struct krigin krigin );
double explNb(struct swarm SW, struct exploit ex );
struct exploit exploitArea(struct swarm SW, struct SS SS, struct param param,
    int update, double explRate[],double rho);
static int compareF (void const *a, void const *b); 
static int compareDist (void const *a, void const *b); 
static int compareLBest (void const *a, void const *b); // For qsort

//struct result localSearch(struct result R,struct problem pb, 
//					struct param param,int g[],struct exploit ex,double rho);

// For variogram and Kriging
struct GammaC variogram(struct position X[],int N,double xMini[DMax],double xMaxi[DMax]);
void matrixDisplay(struct matrix M);
struct matrix matrixInvert(struct matrix M);
struct vector matrixSolve(struct matrix M, struct vector V);
struct vector matrixVect(struct matrix M,struct vector V);
struct model modelOptim(int model,double h[],double g[],int kSup);
struct model modelOptimPSO (int model, struct param param, struct problem problem,double h[],double g[],int kSup);
double modelG(double h, struct model c);
double errorTot(double h[],double g[],int kSup,struct model c); // Fo rKrigin
double errorFC(struct fitness f); // Total erreur (fitness + constraints)
struct position modelFunctOptim (struct position M[],struct GammaC Gamma, double xMin[], double xMax[]);
struct fitness perfModelFunc(struct GammaC G,struct position pos, struct position X[]);

struct position valueAccept(struct position x, int valueNb);
void vectorDisplay(struct vector V);
#include "modelOptimSub.c" 

double paprExhaust(struct problem pb); // Just for tests


// Global variables
double best[RMax][ClassMax]; // To compute the mean best "on the run"
int dLBest; 			// For information 
long double E;			// exp(1). Useful for some test functions
int explForced; // To compute the exploitation rate

int FEmax[ClassMax];
int FEmaxSize;
int FEclass;

struct landscape funct; // When (x,f(x)) is read on a file
int iter;
double nEval;			// Number of fitness evaluations

double NisBD; // pow(param.NisB,1/Dimension);

long double pi;			// Useful for some test functions
int randOption; 		// Kind of randomness to use
int run;
int successFlag[RMax][ClassMax]={0}; // To compute the success rate "in passing"

// For variograms and kriging
double gMax, hMax;

// For PAPR/OFDM
static int aSize=256; 
static float a[16][256];
double Y[256][2];
double paprPenalt=256;

// For Cutting stock
// http://en.wikipedia.org/wiki/Cutting_stock_problem
static float valueList[5]=
{1380, 1820, 1930, 2050, 2150};
static int pieceNb[5]={15,6,3,12,15};
static float toCut=5600;

// File(s);
FILE * f_run;
FILE * f_synth;
FILE * f_expl;
FILE * fLandscape;
FILE * f_init;
FILE * f_init_save;
FILE * fCurve;

FILE * f_vario;
FILE * f_coeff;

FILE *f_papr;

